# Script for for gap-filling and partitioning with multiple constant USTAR thresholds


# CUT_REF FROM FLUXNET WW2020
USTAR_THRESHOLD <- 0.287644
USTAR_SUFFIX <- "CUT_REF"

# # CUT_16 FROM FLUXNET WW2020 (16th percentile from USTAR detection results)
# USTAR_THRESHOLD <- 0.192936
# USTAR_SUFFIX <- "CUT_16"

# # CUT_84 FROM FLUXNET WW2020 (84th percentile from USTAR detection results)
# USTAR_THRESHOLD <- 0.43413
# USTAR_SUFFIX <- "CUT_84"

# START AND END OF DATASET
START_DATE = '1997-01-01 00:30:00'
END_DATE = '2022-08-12 00:00:00'
# START_DATE = '2022-01-01 00:30:00'
# END_DATE = '2022-12-31 00:00:00'


library(ggplot2)
library(REddyProc)
library(caTools)
library(dplyr)
library(viridis)
library(readr)
library(segmented)



# RUN ID
run_id <- format(Sys.time(), "%Y%m%d%H%M%S",tz="GMT")   ## To give unique ID to saved output files



# SOURCE FILE 
# =========== 
file_fluxes_meteo <- "../2_input_data/2-ReddyProcFormat_Dataset_DIIVE-20220826-014627_Original-30T.csv"
output_path <- getwd()
Sys.setenv(TZ = "GMT")



# DATA FROM FILE
# ==============
filedata <- read.csv(file_fluxes_meteo, header = 1)
filedata$TIMESTAMP_END <- as.POSIXct(filedata$TIMESTAMP, format="%Y-%m-%d %H:%M")
summary(filedata)
head(filedata)
colnames(filedata)



# DATA COLUMNS
# ============
EddyData.F <- filedata[FALSE]
EddyData.F$TIMESTAMP <- as.POSIXct(filedata$TIMESTAMP_END, format = '%m/%d/%Y %H:%M', tz = Sys.timezone())

# NEE w/ daytime QCF=0 and 1, and nighttime QCF=0, nighttime was defined as SW_IN < 20
EddyData.F$NEE <- as.numeric(as.character(filedata$NEE_QCF01))
EddyData.F$NEE[EddyData.F$NEE < -50] <- NA  # To remove -9999 missing values
EddyData.F$QCF_NEE <- as.numeric(as.character(filedata$QCF_NEE))
EddyData.F$QCF_NEE[EddyData.F$QCF_NEE < -50] <- NA  # To remove -9999 missing values

EddyData.F$LE <- as.numeric(as.character(filedata$LE_QCF01))
EddyData.F$LE[EddyData.F$LE < -200] <- NA  # To remove -9999 missing values
EddyData.F$QCF_LE <- as.numeric(as.character(filedata$QCF_LE ))
EddyData.F$QCF_LE[EddyData.F$QCF_LE  < -50] <- NA  # To remove -9999 missing values

EddyData.F$Ustar <- as.numeric(as.character(filedata$USTAR))
EddyData.F$Ustar[EddyData.F$Ustar < 0] <- NA  # To remove -9999 missing values

# 1997-2020 Already gap-filled from FLUXNET
EddyData.F$Rg <- as.numeric(as.character(filedata$SW_IN))
EddyData.F$Rg[EddyData.F$Rg < -20] <- NA  # To remove -9999 missing values
EddyData.F$Rg[EddyData.F$Rg < 0] <- 0  # Below zero not accepted by ReddyProc

# 1997-2020 Already gap-filled from FLUXNET
EddyData.F$Tair <- as.numeric(as.character(filedata$TA))
EddyData.F$Tair[EddyData.F$Tair < -20] <- NA  # To remove -9999 missing values

EddyData.F$RH <- as.numeric(as.character(filedata$RH))
EddyData.F$RH[EddyData.F$RH < 0] <- NA  # To remove -9999 missing values
EddyData.F$RH[EddyData.F$RH > 100] <- 100  # Some values are 101

# Already gap-filled from FLUXNET
EddyData.F$VPD_FLUXNET <- as.numeric(as.character(filedata$VPD_FLUXNET))  # Get available VPD 1997-2020
EddyData.F$VPD_FLUXNET[EddyData.F$VPD_FLUXNET < 0] <- NA  # To remove -9999 missing values
EddyData.F$VPD_PI <- fCalcVPDfromRHandTair(EddyData.F$RH, EddyData.F$Tair)  # Calc VPD 2021-2022.08
EddyData.F$VPD <- coalesce(EddyData.F$VPD_FLUXNET, EddyData.F$VPD_PI)

# cut the analysis period to the end of 2020
EddyData.F <- subset (EddyData.F, TIMESTAMP >= as.POSIXct(START_DATE))  # Date with first fluxes
EddyData.F <- subset (EddyData.F, TIMESTAMP <= as.POSIXct(END_DATE))
summary(EddyData.F)



# Initialize R5 reference class
# ============================
EddyProc.C <-sEddyProc$new('CH-DAV',  EddyData.F,
                           c('NEE','LE', 'Rg','Tair','Ustar', 'VPD'), ColPOSIXTime = "TIMESTAMP")   
EddyProc.C$sSetLocationInfo(LatDeg = 46.815333, LongDeg = 9.855917, TimeZoneHour = 1)
str(EddyProc.C)
head(EddyProc.C$sDATA)
head(EddyProc.C$sTEMP)



# GAP FILLING
# ===============

## METEO
## -----
## is already nearly complete
EddyProc.C$sMDSGapFill('Tair', FillAll = FALSE)
EddyProc.C$sMDSGapFill('Rg', FillAll = FALSE)
EddyProc.C$sMDSGapFill('VPD', FillAll = FALSE)


## NEE w/ CONSTANT USTAR THRESHOLD
## -------------------------------

### CUT_REF FROM FLUXNET WW2020
EddyProc.C$sMDSGapFillAfterUstar('NEE',
                                 uStarTh = USTAR_THRESHOLD,
                                 uStarSuffix = USTAR_SUFFIX,
                                 isFlagEntryAfterLowTurbulence = FALSE,
                                 isFilterDayTime = TRUE)

cat("  NEE successfully filled with USTAR filtering!\n")


## LE w/o USTAR THRESHOLD
## ----------------------
## (QC=0 and QC=1)
EddyProc.C$sMDSGapFill('LE', FillAll = TRUE, isVerbose = TRUE)


## ET
## --
## Calculate ET from LE (QC=0 and QC=1)
EddyProc.C$sTEMP$ET_f <- fCalcETfromLE(EddyProc.C$sTEMP$LE_f, EddyProc.C$sTEMP$Tair_f)


summary(EddyProc.C$sTEMP)



# COLLECT DATA AND EXPORT
# =======================
cat("Export data to standard data frame ...")
FilledEddyData.F <- EddyProc.C$sExportResults()
FilledEddyData.F$TIMESTAMP          <-EddyData.F$TIMESTAMP
write.csv(FilledEddyData.F, file = paste("CH-DAV_1997-2022.08_NEE_",USTAR_SUFFIX,"_LE_ET_GAPF_RP-", run_id,".csv",sep="")) 



# PARTITIONING
# ============
EddyProc.C$sMRFluxPartition(suffix = USTAR_SUFFIX)
EddyProc.C$sGLFluxPartition(suffix = USTAR_SUFFIX)



# COLLECT DATA AND EXPORT
# =======================
cat("Export data to standard data frame ...")
FilledEddyData.F <- EddyProc.C$sExportResults()
FilledEddyData.F$TIMESTAMP          <-EddyData.F$TIMESTAMP
write.csv(FilledEddyData.F, file = paste("CH-DAV_1997-2022.08_NEE_",USTAR_SUFFIX,"_LE_ET_GAPF_PART_RP-", run_id,".csv",sep="")) 



# grep("GPP|Reco",names(EddyProc.C$sExportResults()), value = TRUE)
# EddyProc.C$sPlotFingerprintY('GPP_f', Year = 1997)

# grep("_DT",names(EddyProc.C$sExportResults()), value = TRUE)
# EddyProc.C$sPlotFingerprintY('Reco', Year = 2004)



# PLOTS
# =====
# Save fingerprint plots as PDF
EddyProc.C$sPlotFingerprint(paste('NEE', USTAR_SUFFIX, 'f' , sep = "_"))
EddyProc.C$sPlotFingerprint('LE_f')
EddyProc.C$sPlotFingerprint('ET_f')
EddyProc.C$sPlotFingerprint('Tair_f')
EddyProc.C$sPlotFingerprint('Rg_f')
EddyProc.C$sPlotFingerprint('VPD_f')
EddyProc.C$sPlotFingerprint(paste('GPP', USTAR_SUFFIX, 'f' , sep = "_"))
EddyProc.C$sPlotFingerprint(paste('Reco', USTAR_SUFFIX, sep = "_"))
EddyProc.C$sPlotFingerprint(paste('GPP_DT', USTAR_SUFFIX, sep = "_"))
EddyProc.C$sPlotFingerprint(paste('Reco_DT', USTAR_SUFFIX, sep = "_"))


# PLOT PER YEAR
# =============

FilledEddyData.F = FilledEddyData.F %>% mutate(
  Date = as.Date(TIMESTAMP,'%Y-%m-%d'),
  Time = as.character(format(TIMESTAMP, '%H:%M')),
  Year = as.numeric(format(TIMESTAMP,'%Y')))

# NEE, GPP, RECO
# --------------
for (Yr in c(1997:2022))
{
  Year_Filled_Eddy_Data = FilledEddyData.F %>% filter(Year == Yr)
  summary(Year_Filled_Eddy_Data$TIMESTAMP)

  a = ggplot(Year_Filled_Eddy_Data, aes(x = Time, y = Date, fill = NEE_CUT_REF_f)) + geom_tile() + scale_fill_viridis(option = "D") +
    ggtitle('NEE') + theme(axis.text.x=element_blank())

  b = ggplot(Year_Filled_Eddy_Data, aes(x = Time, y = Date, fill = GPP_CUT_REF_f)) + geom_tile() + scale_fill_viridis(option = "D") +
    ggtitle('GPP (nighttime partitioning)') + theme(axis.text.x=element_blank())

  c = ggplot(Year_Filled_Eddy_Data, aes(x = Time, y = Date, fill = Reco_CUT_REF)) + geom_tile() + scale_fill_viridis(option = "D") +
    ggtitle('RECO (nighttime partitioning)') + theme(axis.text.x=element_blank())

  d = ggplot(Year_Filled_Eddy_Data, aes(x = Time, y = Date, fill = GPP_DT_CUT_REF)) + geom_tile() + scale_fill_viridis(option = "D") +
    ggtitle('GPP (daytime partitioning)') + theme(axis.text.x=element_blank())

  e = ggplot(Year_Filled_Eddy_Data, aes(x = Time, y = Date, fill = Reco_DT_CUT_REF)) + geom_tile() + scale_fill_viridis(option = "D") +
    ggtitle('RECO (daytime partitioning)') + theme(axis.text.x=element_blank())

  gr = gridExtra::arrangeGrob(a,b,c,d,e, ncol = 5)

  ggsave(paste(as.character(Yr),"_CH-DAV_NEE_CUT_REF_GPP_RECO_", run_id,".png",sep=""), gr ,height = 200, width = 450, units = 'mm')

  print(summary(Year_Filled_Eddy_Data$TIMESTAMP))
  print(Yr)
}


# NEE, LE, ET
# -----------
for (Yr in c(1997:2022))
{
  Year_Filled_Eddy_Data = FilledEddyData.F %>% filter(Year == Yr)
  summary(Year_Filled_Eddy_Data$TIMESTAMP)

  a = ggplot(Year_Filled_Eddy_Data, aes(x = Time, y = Date, fill = NEE_CUT_REF_f)) + geom_tile() + scale_fill_viridis(option = "D") +
    ggtitle('NEE') + theme(axis.text.x=element_blank())

  b = ggplot(Year_Filled_Eddy_Data, aes(x = Time, y = Date, fill = LE_f)) + geom_tile() + scale_fill_viridis(option = "D") +
    ggtitle('LE') + theme(axis.text.x=element_blank())

  c = ggplot(Year_Filled_Eddy_Data, aes(x = Time, y = Date, fill = ET_f)) + geom_tile() + scale_fill_viridis(option = "D") +
    ggtitle('ET') + theme(axis.text.x=element_blank())

  gr = gridExtra::arrangeGrob(a,b,c, ncol = 3)

  ggsave(paste(as.character(Yr),"_CH-DAV_NEE_CUT_REF_LE_ET_", run_id,".png",sep=""), gr ,height = 200, width = 450, units = 'mm')

  print(summary(Year_Filled_Eddy_Data$TIMESTAMP))
  print(Yr)
}



# SW_IN, TA, VPD
# --------------
for (Yr in c(1997:2022))
{
  Year_Filled_Eddy_Data = FilledEddyData.F %>% filter(Year == Yr)
  summary(Year_Filled_Eddy_Data$TIMESTAMP)

  a = ggplot(Year_Filled_Eddy_Data, aes(x = Time, y = Date, fill = Rg_f)) + geom_tile() + scale_fill_viridis(option = "D") +
    ggtitle('SW_IN') + theme(axis.text.x=element_blank())

  b = ggplot(Year_Filled_Eddy_Data, aes(x = Time, y = Date, fill = Tair_f)) + geom_tile() + scale_fill_viridis(option = "D") +
    ggtitle('TA') + theme(axis.text.x=element_blank())

  c = ggplot(Year_Filled_Eddy_Data, aes(x = Time, y = Date, fill = VPD_f)) + geom_tile() + scale_fill_viridis(option = "D") +
    ggtitle('VPD') + theme(axis.text.x=element_blank())

  gr = gridExtra::arrangeGrob(a,b,c, ncol = 3)

  ggsave(paste(as.character(Yr),"_CH-DAV_SWIN_TA_VPD_", run_id,".png",sep=""), gr ,height = 200, width = 450, units = 'mm')

  print(summary(Year_Filled_Eddy_Data$TIMESTAMP))
  print(Yr)
}







# DAILY AGGREGATION
# =================

## While daily aggregation only choosing QC = 0 and 1
FilledEddyData.F_daily = FilledEddyData.F %>% group_by(Date) %>%
  summarise(NEE_CUT_f_mean = mean(NEE_CUT_REF_f),
            GPP_CUT_f_mean = mean(GPP_CUT_REF_f),
            Reco_CUT_mean = mean(Reco_CUT_REF),
            GPP_DT_CUT_mean = mean(GPP_DT_CUT_REF),
            Reco_DT_CUT_mean = mean(Reco_DT_CUT_REF),
            LE_f_mean = mean(LE_f),
            ET_f_mean = mean(ET_f),
            VPD_f_mean = mean(VPD_f),
            TA_mean = mean(Tair_f),
            SW_IN_mean = mean(Rg_f)) %>%
  mutate(Year = as.numeric(format(Date, '%Y')), Month = as.numeric(format(Date,'%m')), DOY = as.numeric(format(Date,'%j')))

summary(FilledEddyData.F_daily)

# write.csv(FilledEddyData.F_daily, 'CH_DAV_LE_ET_VPD_TA_SW_IN_daily.csv', row.names = FALSE)

## Plotting daily Fluxes

a = ggplot(FilledEddyData.F_daily, aes(x = DOY, y = Year, fill = NEE_CUT_f_mean)) + geom_tile() + scale_fill_viridis(option = "D") +
  ggtitle('NEE_f_mean daily gapfilled') + theme(axis.text.x=element_blank()) + theme_bw()

b = ggplot(FilledEddyData.F_daily, aes(x = DOY, y = Year, fill = GPP_CUT_f_mean)) + geom_tile() + scale_fill_viridis(option = "D") +
  ggtitle('GPP_f_mean daily gapfilled') + theme(axis.text.x=element_blank()) + theme_bw()

c = ggplot(FilledEddyData.F_daily, aes(x = DOY, y = Year, fill = Reco_CUT_mean)) + geom_tile() + scale_fill_viridis(option = "D") +
  ggtitle('RECO_mean daily gapfilled') + theme(axis.text.x=element_blank()) + theme_bw()

d = ggplot(FilledEddyData.F_daily, aes(x = DOY, y = Year, fill = GPP_DT_CUT_mean)) + geom_tile() + scale_fill_viridis(option = "D") +
  ggtitle('GPP_DT_mean daily gapfilled') + theme(axis.text.x=element_blank()) + theme_bw()

e = ggplot(FilledEddyData.F_daily, aes(x = DOY, y = Year, fill = Reco_DT_CUT_mean)) + geom_tile() + scale_fill_viridis(option = "D") +
  ggtitle('RECO_DT_mean daily gapfilled') + theme(axis.text.x=element_blank()) + theme_bw()

f = ggplot(FilledEddyData.F_daily, aes(x = DOY, y = Year, fill = LE_f_mean)) + geom_tile() + scale_fill_viridis(option = "D") +
  ggtitle('LE_f_mean daily gapfilled') + theme(axis.text.x=element_blank()) + theme_bw()

g = ggplot(FilledEddyData.F_daily, aes(x = DOY, y = Year, fill = ET_f_mean)) + geom_tile() + scale_fill_viridis(option = "D") +
  ggtitle('ET_f_mean daily gapfilled') + theme(axis.text.x=element_blank()) + theme_bw()

h = ggplot(FilledEddyData.F_daily, aes(x = DOY, y = Year, fill = VPD_f_mean)) + geom_tile() + scale_fill_viridis(option = "D") +
  ggtitle('VPD_f_mean daily gapfilled') + theme(axis.text.x=element_blank()) + theme_bw()

i = ggplot(FilledEddyData.F_daily, aes(x = DOY, y = Year, fill = TA_mean)) + geom_tile() + scale_fill_viridis(option = "D") +
  ggtitle('TA_mean daily gapfilled') + theme(axis.text.x=element_blank()) + theme_bw()

j = ggplot(FilledEddyData.F_daily, aes(x = DOY, y = Year, fill = SW_IN_mean)) + geom_tile() + scale_fill_viridis(option = "D") +
  ggtitle('SW_IN_mean daily gapfilled') + theme(axis.text.x=element_blank()) + theme_bw()

gr = gridExtra::arrangeGrob(a,b,c,d,e,f,g,h,i,j, ncol = 5)

ggsave("CH-DAV_2020_DOY_Daily.png", gr ,height = 300, width =600, units = 'mm')


#############------------------------------ End of Script --------------------------------------------------------------------##################
### Thank you for your patience.
